#!/usr/bin/env python
# Copyright (C) 2013 Yahoo! Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron.v2_0 import quota as test_quota
from neutronclient.tests.unit import test_cli20


class CLITestV20Quota(test_cli20.CLITestV20Base):
    def test_show_quota(self):
        resource = 'quota'
        cmd = test_quota.ShowQuota(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--tenant-id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args)

    def test_update_quota(self):
        resource = 'quota'
        cmd = test_quota.UpdateQuota(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--tenant-id', self.test_id, '--network', 'test']
        self.assertRaises(
            exceptions.CommandError, self._test_update_resource,
            resource, cmd, self.test_id, args=args,
            extrafields={'network': 'new'})

    def test_delete_quota_get_parser(self):
        cmd = test_cli20.MyApp(sys.stdout)
        test_quota.DeleteQuota(cmd, None).get_parser(cmd)

    def test_show_quota_positional(self):
        resource = 'quota'
        cmd = test_quota.ShowQuota(
            test_cli20.MyApp(sys.stdout), None)
        args = [self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args)

    def test_update_quota_positional(self):
        resource = 'quota'
        cmd = test_quota.UpdateQuota(
            test_cli20.MyApp(sys.stdout), None)
        args = [self.test_id, '--network', 'test']
        self.assertRaises(
            exceptions.CommandError, self._test_update_resource,
            resource, cmd, self.test_id, args=args,
            extrafields={'network': 'new'})

    def test_show_quota_default(self):
        resource = 'quota'
        cmd = test_quota.ShowQuotaDefault(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--tenant-id', self.test_id]
        expected_res = {'quota': {'port': 50, 'network': 10, 'subnet': 10}}
        resstr = self.client.serialize(expected_res)
        path = getattr(self.client, "quota_default_path")
        return_tup = (test_cli20.MyResp(200), resstr)
        with mock.patch.object(cmd, 'get_client',
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, 'request',
                                  return_value=return_tup) as mock_request:
            cmd_parser = cmd.get_parser("test_" + resource)
            parsed_args = cmd_parser.parse_args(args)
            cmd.run(parsed_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path % self.test_id), 'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn('network', _str)
        self.assertIn('subnet', _str)
        self.assertIn('port', _str)
        self.assertNotIn('subnetpool', _str)

    def test_update_quota_noargs(self):
        resource = 'quota'
        cmd = test_quota.UpdateQuota(test_cli20.MyApp(sys.stdout), None)
        args = [self.test_id]
        self.assertRaises(exceptions.CommandError, self._test_update_resource,
                          resource, cmd, self.test_id, args=args,
                          extrafields=None)



# Copyright (c) 2016 Intel Corporation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.qos import minimum_bandwidth_rule as bw_rule
from neutronclient.tests.unit import test_cli20


class CLITestV20QoSMinimumBandwidthRuleJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['minimum_bandwidth_rule']

    def setUp(self):
        super(CLITestV20QoSMinimumBandwidthRuleJSON, self).setUp()
        self.res = 'minimum_bandwidth_rule'
        self.cmd_res = 'qos_minimum_bandwidth_rule'
        self.ress = self.res + 's'
        self.cmd_ress = self.cmd_res + 's'

    def test_create_minimum_bandwidth_rule_min_kbps_only(self):
        cmd = bw_rule.CreateQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        min_kbps = '1500'
        policy_id = 'policy_id'
        args = ['--min-kbps', min_kbps,
                policy_id]
        position_names = ['min_kbps']
        position_values = [min_kbps]
        self.assertRaises(SystemExit, self._test_create_resource,
                          self.res, cmd, '', my_id, args,
                          position_names, position_values,
                          cmd_resource=self.cmd_res,
                          parent_id=policy_id,
                          no_api_call=True)

    def test_create_minimum_bandwidth_rule_direction_only(self):
        cmd = bw_rule.CreateQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        direction = 'egress'
        policy_id = 'policy_id'
        args = ['--direction', direction,
                policy_id]
        position_names = ['direction']
        position_values = [direction]
        self.assertRaises(SystemExit, self._test_create_resource,
                          self.res, cmd, '', my_id, args,
                          position_names, position_values,
                          cmd_resource=self.cmd_res,
                          parent_id=policy_id,
                          no_api_call=True)

    def test_create_minimum_bandwidth_rule_none(self):
        cmd = bw_rule.CreateQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        policy_id = 'policy_id'
        args = [policy_id]
        position_names = []
        position_values = []
        self.assertRaises(SystemExit, self._test_create_resource,
                          self.res, cmd, '', my_id, args,
                          position_names, position_values,
                          cmd_resource=self.cmd_res,
                          parent_id=policy_id,
                          no_api_call=True)

    def test_create_minimum_bandwidth_rule_all(self):
        cmd = bw_rule.CreateQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        min_kbps = '1500'
        direction = 'egress'
        policy_id = 'policy_id'
        args = ['--min-kbps', min_kbps,
                '--direction', direction,
                policy_id]
        position_names = ['direction', 'min_kbps']
        position_values = [direction, min_kbps]
        self._test_create_resource(self.res, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_update_minimum_bandwidth_rule(self):
        cmd = bw_rule.UpdateQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        min_kbps = '1200'
        direction = 'egress'
        policy_id = 'policy_id'
        args = ['--min-kbps', min_kbps,
                '--direction', direction,
                my_id, policy_id]
        self._test_update_resource(self.res, cmd, my_id, args,
                                   {'min_kbps': min_kbps,
                                    'direction': direction},
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_delete_minimum_bandwidth_rule(self):
        cmd = bw_rule.DeleteQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        policy_id = 'policy_id'
        args = [my_id, policy_id]
        self._test_delete_resource(self.res, cmd, my_id, args,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_show_minimum_bandwidth_rule(self):
        cmd = bw_rule.ShowQoSMinimumBandwidthRule(
            test_cli20.MyApp(sys.stdout), None)
        policy_id = 'policy_id'
        args = [self.test_id, policy_id]
        self._test_show_resource(self.res, cmd, self.test_id, args,
                                 [], cmd_resource=self.cmd_res,
                                 parent_id=policy_id)

    def test_list_minimum_bandwidth_rule(self):
        cmd = bw_rule.ListQoSMinimumBandwidthRules(
            test_cli20.MyApp(sys.stdout), None)
        policy_id = 'policy_id'
        args = [policy_id]
        contents = [{'name': 'rule1', 'min-kbps': 1000, 'direction': 'egress'}]
        self._test_list_resources(self.ress, cmd, parent_id=policy_id,
                                  cmd_resources=self.cmd_ress,
                                  base_args=args, response_contents=contents)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient.common import utils


API_NAME = 'network'
API_VERSIONS = {
    '2.0': 'neutronclient.v2_0.client.Client',
    '2': 'neutronclient.v2_0.client.Client',
}


def make_client(instance):
    """Returns an neutron client."""
    neutron_client = utils.get_client_class(
        API_NAME,
        instance._api_version,
        API_VERSIONS,
    )
    instance.initialize()
    url = instance._url
    url = url.rstrip("/")
    client = neutron_client(username=instance._username,
                            project_name=instance._project_name,
                            password=instance._password,
                            region_name=instance._region_name,
                            auth_url=instance._auth_url,
                            endpoint_url=url,
                            endpoint_type=instance._endpoint_type,
                            token=instance._token,
                            auth_strategy=instance._auth_strategy,
                            insecure=instance._insecure,
                            ca_cert=instance._ca_cert,
                            retries=instance._retries,
                            raise_errors=instance._raise_errors,
                            session=instance._session,
                            auth=instance._auth)
    return client


def Client(api_version, *args, **kwargs):
    """Return an neutron client.

    @param api_version: only 2.0 is supported now
    """
    neutron_client = utils.get_client_class(
        API_NAME,
        api_version,
        API_VERSIONS,
    )
    return neutron_client(*args, **kwargs)



# Copyright 2015 Huawei Technologies India Pvt Ltd, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import os

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronv20


def get_qos_policy_id(client, policy_id_or_name):
    _policy_id = neutronv20.find_resourceid_by_name_or_id(
        client, 'policy', policy_id_or_name, cmd_resource='qos_policy')
    return _policy_id


class CreateQosPolicyMixin(object):
    def add_arguments_qos_policy(self, parser):
        qos_policy_args = parser.add_mutually_exclusive_group()
        qos_policy_args.add_argument(
            '--qos-policy',
            help=_('ID or name of the QoS policy that should'
                   'be attached to the resource.'))
        return qos_policy_args

    def args2body_qos_policy(self, parsed_args, resource):
        if parsed_args.qos_policy:
            _policy_id = get_qos_policy_id(self.get_client(),
                                           parsed_args.qos_policy)
            resource['qos_policy_id'] = _policy_id


class UpdateQosPolicyMixin(CreateQosPolicyMixin):
    def add_arguments_qos_policy(self, parser):
        qos_policy_args = (super(UpdateQosPolicyMixin, self).
                           add_arguments_qos_policy(parser))
        qos_policy_args.add_argument(
            '--no-qos-policy',
            action='store_true',
            help=_('Detach QoS policy from the resource.'))
        return qos_policy_args

    def args2body_qos_policy(self, parsed_args, resource):
        super(UpdateQosPolicyMixin, self).args2body_qos_policy(parsed_args,
                                                               resource)
        if parsed_args.no_qos_policy:
            resource['qos_policy_id'] = None


class ListQoSPolicy(neutronv20.ListCommand):
    """List QoS policies that belong to a given tenant connection."""

    resource = 'policy'
    shadow_resource = 'qos_policy'
    list_columns = ['id', 'name']
    pagination_support = True
    sorting_support = True


class ShowQoSPolicy(neutronv20.ShowCommand):
    """Show information of a given qos policy."""

    resource = 'policy'
    shadow_resource = 'qos_policy'

    def format_output_data(self, data):
        rules = []
        for rule in data['policy'].get('rules', []):
            rules.append("%s (type: %s)" % (rule['id'], rule['type']))
        data['policy']['rules'] = os.linesep.join(rules)

        super(ShowQoSPolicy, self).format_output_data(data)


class CreateQoSPolicy(neutronv20.CreateCommand):
    """Create a qos policy."""

    resource = 'policy'
    shadow_resource = 'qos_policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name of the QoS policy to be created.'))
        parser.add_argument(
            '--description',
            help=_('Description of the QoS policy to be created.'))
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Accessible by other tenants. '
                   'Set shared to True (default is False).'))

    def args2body(self, parsed_args):
        body = {'name': parsed_args.name}
        if parsed_args.description:
            body['description'] = parsed_args.description
        if parsed_args.shared:
            body['shared'] = parsed_args.shared
        if parsed_args.tenant_id:
            body['tenant_id'] = parsed_args.tenant_id
        return {self.resource: body}


class UpdateQoSPolicy(neutronv20.UpdateCommand):
    """Update a given qos policy."""

    resource = 'policy'
    shadow_resource = 'qos_policy'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name',
            help=_('Name of the QoS policy.'))
        parser.add_argument(
            '--description',
            help=_('Description of the QoS policy.'))
        shared_group = parser.add_mutually_exclusive_group()
        shared_group.add_argument(
            '--shared',
            action='store_true',
            help=_('Accessible by other tenants. '
                   'Set shared to True (default is False).'))
        shared_group.add_argument(
            '--no-shared',
            action='store_true',
            help=_('Not accessible by other tenants. '
                   'Set shared to False.'))

    def args2body(self, parsed_args):
        body = {}
        if parsed_args.name:
            body['name'] = parsed_args.name
        if parsed_args.description:
            body['description'] = parsed_args.description
        if parsed_args.shared:
            body['shared'] = True
        if parsed_args.no_shared:
            body['shared'] = False

        return {self.resource: body}


class DeleteQoSPolicy(neutronv20.DeleteCommand):
    """Delete a given qos policy."""

    resource = 'policy'
    shadow_resource = 'qos_policy'



# Copyright 2015 Hewlett-Packard Development Company, L.P
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutronclient.tests.functional import base


class SubnetCreateNeutronClientCLITest(base.ClientTestBase):

    def test_create_subnet_net_name_first(self):
        self.neutron('net-create', params='netwrk-1')
        self.addCleanup(self.neutron, 'net-delete netwrk-1')
        self.neutron('subnet-create netwrk-1',
                     params='--name fake --gateway 192.168.51.1 '
                            '192.168.51.0/24')
        self.addCleanup(self.neutron, 'subnet-delete fake')
        subnet_list = self.parser.listing(self.neutron('subnet-list'))
        self.assertTableStruct(subnet_list, ['id', 'name', 'cidr',
                                             'allocation_pools'])
        found = False
        for row in subnet_list:
            if row.get('name') == 'fake':
                found = True
                break
        if not found:
            self.fail('Created subnet not found in list')



# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc import utils as nc_osc_utils
from neutronclient.osc.v2.networking_bgpvpn import constants

LOG = logging.getLogger(__name__)


class CreateBgpvpnResAssoc(command.ShowOne):
    """Create a BGP VPN resource association"""
    _action = 'create'

    def get_parser(self, prog_name):
        parser = super(CreateBgpvpnResAssoc, self).get_parser(prog_name)
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=(_("BGP VPN to apply the %s association (name or ID)") %
                  self._assoc_res_name),
        )
        parser.add_argument(
            'resource',
            metavar="<%s>" % self._assoc_res_name,
            help=(_("%s to associate the BGP VPN (name or ID)") %
                  self._assoc_res_name.capitalize()),
        )

        get_common_parser = getattr(self, '_get_common_parser', None)
        if callable(get_common_parser):
            get_common_parser(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        create_method = getattr(
            client, 'create_bgpvpn_%s_assoc' % self._assoc_res_name)
        bgpvpn = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)
        assoc_res = client.find_resource(self._assoc_res_name,
                                         parsed_args.resource)
        body = {
            self._resource: {
                '%s_id' % self._assoc_res_name: assoc_res['id'],
            },
        }
        if 'project' in parsed_args and parsed_args.project is not None:
            project_id = nc_osc_utils.find_project(
                self.app.client_manager.identity,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            body[self._resource]['tenant_id'] = project_id

        arg2body = getattr(self, '_args2body', None)
        if callable(arg2body):
            body[self._resource].update(
                arg2body(bgpvpn['id'], parsed_args)[self._resource])

        obj = create_method(bgpvpn['id'], body)[self._resource]
        transform = getattr(self, '_transform_resource', None)
        if callable(transform):
            transform(obj)
        columns, display_columns = column_util.get_columns(obj, self._attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=self._formatters)
        return display_columns, data


class SetBgpvpnResAssoc(command.Command):
    """Set BGP VPN resource association properties"""
    _action = 'set'

    def get_parser(self, prog_name):
        parser = super(SetBgpvpnResAssoc, self).get_parser(prog_name)
        parser.add_argument(
            'resource_association_id',
            metavar="<%s association ID>" % self._assoc_res_name,
            help=(_("%s association ID to update") %
                  self._assoc_res_name.capitalize()),
        )
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=(_("BGP VPN the %s association belongs to (name or ID)") %
                  self._assoc_res_name),
        )

        get_common_parser = getattr(self, '_get_common_parser', None)
        if callable(get_common_parser):
            get_common_parser(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        update_method = getattr(
            client, 'update_bgpvpn_%s_assoc' % self._assoc_res_name)
        bgpvpn = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)
        arg2body = getattr(self, '_args2body', None)
        if callable(arg2body):
            body = arg2body(bgpvpn['id'], parsed_args)
            update_method(bgpvpn['id'], parsed_args.resource_association_id,
                          body)


class UnsetBgpvpnResAssoc(SetBgpvpnResAssoc):
    """Unset BGP VPN resource association properties"""
    _action = 'unset'


class DeleteBgpvpnResAssoc(command.Command):
    """Remove a BGP VPN resource association(s) for a given BGP VPN"""

    def get_parser(self, prog_name):
        parser = super(DeleteBgpvpnResAssoc, self).get_parser(prog_name)
        parser.add_argument(
            'resource_association_ids',
            metavar="<%s association ID>" % self._assoc_res_name,
            nargs="+",
            help=(_("%s association ID(s) to remove") %
                  self._assoc_res_name.capitalize()),
        )
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=(_("BGP VPN the %s association belongs to (name or ID)") %
                  self._assoc_res_name),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        delete_method = getattr(
            client, 'delete_bgpvpn_%s_assoc' % self._assoc_res_name)
        bgpvpn = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)
        fails = 0
        for id in parsed_args.resource_association_ids:
            try:
                delete_method(bgpvpn['id'], id)
                LOG.warning(
                    "%(assoc_res_name)s association %(id)s deleted",
                    {'assoc_res_name': self._assoc_res_name.capitalize(),
                     'id': id})
            except Exception as e:
                fails += 1
                LOG.error("Failed to delete %(assoc_res_name)s "
                          "association with ID '%(id)s': %(e)s",
                          {'assoc_res_name': self._assoc_res_name,
                           'id': id,
                           'e': e})
        if fails > 0:
            msg = (_("Failed to delete %(fails)s of %(total)s "
                     "%(assoc_res_name)s BGP VPN association(s).") %
                   {'fails': fails,
                    'total': len(parsed_args.resource_association_ids),
                    'assoc_res_name': self._assoc_res_name})
            raise exceptions.CommandError(msg)


class ListBgpvpnResAssoc(command.Lister):
    """List BGP VPN resource associations for a given BGP VPN"""

    def get_parser(self, prog_name):
        parser = super(ListBgpvpnResAssoc, self).get_parser(prog_name)
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=_("BGP VPN listed associations belong to (name or ID)"),
        )
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output"),
        )
        parser.add_argument(
            '--property',
            metavar="<key=value>",
            help=_("Filter property to apply on returned BGP VPNs (repeat to "
                   "filter on multiple properties)"),
            action=parseractions.KeyValueAction,
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        list_method = getattr(client,
                              'list_bgpvpn_%s_assocs' % self._assoc_res_name)
        bgpvpn = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)
        params = {}
        if parsed_args.property:
            params.update(parsed_args.property)
        objs = list_method(bgpvpn['id'],
                           retrieve_all=True, **params)[self._resource_plural]
        transform = getattr(self, '_transform_resource', None)
        if callable(transform):
            [transform(obj) for obj in objs]
        headers, columns = column_util.get_column_definitions(
            self._attr_map, long_listing=parsed_args.long)
        return (headers, (osc_utils.get_dict_properties(
            s, columns, formatters=self._formatters) for s in objs))


class ShowBgpvpnResAssoc(command.ShowOne):
    """Show information of a given BGP VPN resource association"""

    def get_parser(self, prog_name):
        parser = super(ShowBgpvpnResAssoc, self).get_parser(prog_name)
        parser.add_argument(
            'resource_association_id',
            metavar="<%s association ID>" % self._assoc_res_name,
            help=(_("%s association ID to look up") %
                  self._assoc_res_name.capitalize()),
        )
        parser.add_argument(
            'bgpvpn',
            metavar="<bgpvpn>",
            help=_("BGP VPN the association belongs to (name or ID)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        show_method = getattr(client,
                              'show_bgpvpn_%s_assoc' % self._assoc_res_name)
        bgpvpn = client.find_resource(constants.BGPVPN, parsed_args.bgpvpn)
        assoc = client.find_resource_by_id(
            self._resource,
            parsed_args.resource_association_id,
            cmd_resource='bgpvpn_%s_assoc' % self._assoc_res_name,
            parent_id=bgpvpn['id'])
        obj = show_method(bgpvpn['id'], assoc['id'])[self._resource]
        transform = getattr(self, '_transform_resource', None)
        if callable(transform):
            transform(obj)
        columns, display_columns = column_util.get_columns(obj, self._attr_map)
        data = osc_utils.get_dict_properties(obj, columns,
                                             formatters=self._formatters)
        return display_columns, data



# Copyright (c) 2016 Juniper networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc.v2.networking_bgpvpn import constants
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    CreateBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    DeleteBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    ListBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    SetBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    ShowBgpvpnResAssoc
from neutronclient.osc.v2.networking_bgpvpn.resource_association import\
    UnsetBgpvpnResAssoc


class BgpvpnRouterAssoc(object):
    _assoc_res_name = constants.ROUTER_RESOURCE_NAME
    _resource = constants.ROUTER_ASSOC
    _resource_plural = constants.ROUTER_ASSOCS

    _attr_map = (
        ('id', 'ID', column_util.LIST_BOTH),
        ('tenant_id', 'Project', column_util.LIST_LONG_ONLY),
        ('%s_id' % _assoc_res_name, '%s ID' % _assoc_res_name.capitalize(),
         column_util.LIST_BOTH),
        ('advertise_extra_routes', 'Advertise extra routes',
         column_util.LIST_LONG_ONLY),
    )
    _formatters = {}

    def _get_common_parser(self, parser):
        """Adds to parser arguments common to create, set and unset commands.

        :params ArgumentParser parser: argparse object contains all command's
                                       arguments
        """
        ADVERTISE_ROUTES = _("Routes will be advertised to the "
                             "BGP VPN%s") % (
                                 _(' (default)') if self._action == 'create'
                                 else "")
        NOT_ADVERTISE_ROUTES = _("Routes from the router will not be "
                                 "advertised to the BGP VPN")

        group_advertise_extra_routes = parser.add_mutually_exclusive_group()
        group_advertise_extra_routes.add_argument(
            '--advertise_extra_routes',
            action='store_true',
            help=NOT_ADVERTISE_ROUTES if self._action == 'unset'
            else ADVERTISE_ROUTES,
        )
        group_advertise_extra_routes.add_argument(
            '--no-advertise_extra_routes',
            action='store_true',
            help=ADVERTISE_ROUTES if self._action == 'unset'
            else NOT_ADVERTISE_ROUTES,
        )

    def _args2body(self, _, args):
        attrs = {}

        if args.advertise_extra_routes:
            attrs['advertise_extra_routes'] = self._action != 'unset'
        elif args.no_advertise_extra_routes:
            attrs['advertise_extra_routes'] = self._action == 'unset'

        return {self._resource: attrs}


class CreateBgpvpnRouterAssoc(BgpvpnRouterAssoc, CreateBgpvpnResAssoc):
    _description = _("Create a BGP VPN router association")
    pass


class SetBgpvpnRouterAssoc(BgpvpnRouterAssoc, SetBgpvpnResAssoc):
    _description = _("Set BGP VPN router association properties")


class UnsetBgpvpnRouterAssoc(BgpvpnRouterAssoc, UnsetBgpvpnResAssoc):
    _description = _("Unset BGP VPN router association properties")


class DeleteBgpvpnRouterAssoc(BgpvpnRouterAssoc, DeleteBgpvpnResAssoc):
    _description = _("Delete a BGP VPN router association(s) for a given BGP "
                     "VPN")
    pass


class ListBgpvpnRouterAssoc(BgpvpnRouterAssoc, ListBgpvpnResAssoc):
    _description = _("List BGP VPN router associations for a given BGP VPN")
    pass


class ShowBgpvpnRouterAssoc(BgpvpnRouterAssoc, ShowBgpvpnResAssoc):
    _description = _("Show information of a given BGP VPN router association")
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from cliff import show

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


class ListIpAvailability(neutronV20.ListCommand):
    """List IP usage of networks"""

    resource = 'network_ip_availability'
    resource_plural = 'network_ip_availabilities'
    list_columns = ['network_id', 'network_name', 'total_ips', 'used_ips']
    paginations_support = True
    sorting_support = True

    filter_attrs = [
        {'name': 'ip_version',
         'help': _('Returns IP availability for the network subnets '
                   'with a given IP version. Default: 4'),
         'argparse_kwargs': {'type': int,
                             'choices': [4, 6],
                             'default': 4}
         },
        {'name': 'network_id',
         'help': _('Returns IP availability for the network '
                   'matching a given network ID.')},
        {'name': 'network_name',
         'help': _('Returns IP availability for the network '
                   'matching a given name.')},
        {'name': 'tenant_id',
         'help': _('Returns IP availability for the networks '
                   'with a given tenant ID.')},
    ]


class ShowIpAvailability(neutronV20.NeutronCommand, show.ShowOne):
    """Show IP usage of specific network"""

    resource = 'network_ip_availability'

    def get_parser(self, prog_name):
        parser = super(ShowIpAvailability, self).get_parser(prog_name)
        parser.add_argument(
            'network_id', metavar='NETWORK',
            help=_('ID or name of network to look up.'))
        return parser

    def take_action(self, parsed_args):
        self.log.debug('run(%s)', parsed_args)
        neutron_client = self.get_client()
        _id = neutronV20.find_resourceid_by_name_or_id(
            neutron_client, 'network', parsed_args.network_id)
        data = neutron_client.show_network_ip_availability(_id)
        self.format_output_data(data)
        resource = data[self.resource]
        if self.resource in data:
            return zip(*sorted(resource.items()))
        else:
            return None



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.flavor import flavor
from neutronclient.tests.unit import test_cli20


class CLITestV20FlavorJSON(test_cli20.CLITestV20Base):

    def setUp(self):
        """Prepare test environment."""
        super(CLITestV20FlavorJSON, self).setUp(plurals={'flavors': 'flavor'})
        self.register_non_admin_status_resource('flavor')
        self.register_non_admin_status_resource('service_profile')

    def test_create_flavor_with_missing_params(self):
        """Create test flavor with missing parameters."""
        resource = 'flavor'
        cmd = flavor.CreateFlavor(
            test_cli20.MyApp(sys.stdout), None)
        name = 'Test flavor'
        myid = 'myid'
        position_names = []
        position_values = []
        args = []
        self.assertRaises(
            SystemExit, self._test_create_resource,
            resource, cmd, name, myid, args, position_names, position_values)

    def test_create_flavor_with_mandatory_params(self):
        """Create test flavor with minimal parameters."""
        resource = 'flavor'
        cmd = flavor.CreateFlavor(
            test_cli20.MyApp(sys.stdout), None)
        name = 'Test flavor'
        myid = 'myid'
        service_type = 'DUMMY'
        # Defaults are returned in body
        position_names = ['name', 'service_type']
        position_values = [name, service_type]
        args = [name, service_type]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_flavor_with_optional_params(self):
        """Create test flavor including optional parameters."""
        resource = 'flavor'
        cmd = flavor.CreateFlavor(
            test_cli20.MyApp(sys.stdout), None)
        name = 'Test flavor'
        myid = 'myid'
        service_type = 'DUMMY'
        description = 'Test description'
        position_names = ['name', 'service_type', 'description', 'enabled']
        position_values = [name, service_type, description, 'False']
        args = [name, service_type,
                '--description', description,
                '--enabled=False']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_delete_flavor(self):
        """Delete flavor."""
        resource = 'flavor'
        cmd = flavor.DeleteFlavor(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_list_flavors(self):
        """List flavors test."""
        resources = 'flavors'
        cmd = flavor.ListFlavor(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_flavors_with_pagination(self):
        """List flavors test with pagination."""
        resources = 'flavors'
        cmd = flavor.ListFlavor(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_flavors_with_sort(self):
        """List flavors test with sorting by name and id."""
        resources = 'flavors'
        cmd = flavor.ListFlavor(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_show_flavor(self):
        """Show flavor test."""
        resource = 'flavor'
        cmd = flavor.ShowFlavor(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_update_flavor_with_name(self):
        """Update flavor test."""
        resource = 'flavor'
        cmd = flavor.UpdateFlavor(
            test_cli20.MyApp(sys.stdout), None)
        newname = 'Test New Name'
        newdescription = 'New Description'
        args = ['--name', newname,
                '--description', newdescription,
                '--enabled', 'False', self.test_id]
        self._test_update_resource(resource, cmd, self.test_id, args,
                                   {'name': newname,
                                    'description': newdescription,
                                    'enabled': 'False'})

    def test_associate_flavor(self):
        """Associate flavor test."""
        resource = 'service_profile'
        cmd = flavor.AssociateFlavor(test_cli20.MyApp(sys.stdout), None)
        flavor_id = 'flavor-id'
        profile_id = 'profile-id'
        name = ''
        args = [flavor_id, profile_id]
        position_names = ['id']
        position_values = [profile_id]
        self._test_create_resource(resource, cmd, name, profile_id, args,
                                   position_names, position_values,
                                   cmd_resource='flavor_profile_binding',
                                   parent_id=flavor_id)

    def test_disassociate_flavor(self):
        """Disassociate flavor test."""
        resource = 'flavor_profile_binding'
        cmd = flavor.DisassociateFlavor(test_cli20.MyApp(sys.stdout), None)
        flavor_id = 'flavor-id'
        profile_id = 'profile-id'
        args = [flavor_id, profile_id]
        self._test_delete_resource(resource, cmd, profile_id, args,
                                   parent_id=flavor_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from tempest.lib import exceptions

from neutronclient.tests.functional import base


class SimpleReadOnlyNeutronClientTest(base.ClientTestBase):

    """This is a first pass at a simple read only python-neutronclient test.

    This only exercises client commands that are read only.
    This should test commands:
    * as a regular user
    * as a admin user
    * with and without optional parameters
    * initially just check return codes, and later test command outputs
    """

    def test_admin_fake_action(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.neutron,
                          'this-does-neutron-exist')

    # NOTE(mestery): Commands in order listed in 'neutron help'

    # Optional arguments:

    def test_neutron_fake_action(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.neutron,
                          'this-does-not-exist')

    def test_neutron_net_list(self):
        net_list = self.parser.listing(self.neutron('net-list'))
        self.assertTableStruct(net_list, ['id', 'name', 'subnets'])

    def test_neutron_ext_list(self):
        ext = self.parser.listing(self.neutron('ext-list'))
        self.assertTableStruct(ext, ['alias', 'name'])

    def test_neutron_dhcp_agent_list_hosting_net(self):
        self.neutron('dhcp-agent-list-hosting-net',
                     params='private')

    def test_neutron_agent_list(self):
        agents = self.parser.listing(self.neutron('agent-list'))
        field_names = ['id', 'agent_type', 'host', 'alive', 'admin_state_up']
        self.assertTableStruct(agents, field_names)

    def test_neutron_floatingip_list(self):
        self.neutron('floatingip-list')

    def test_neutron_meter_label_list(self):
        self.neutron('meter-label-list')

    def test_neutron_meter_label_rule_list(self):
        self.neutron('meter-label-rule-list')

    def test_neutron_net_external_list(self):
        net_ext_list = self.parser.listing(self.neutron('net-external-list'))
        self.assertTableStruct(net_ext_list, ['id', 'name', 'subnets'])

    def test_neutron_port_list(self):
        port_list = self.parser.listing(self.neutron('port-list'))
        self.assertTableStruct(port_list, ['id', 'name', 'mac_address',
                                           'fixed_ips'])

    def test_neutron_quota_list(self):
        self.neutron('quota-list')

    def test_neutron_router_list(self):
        router_list = self.parser.listing(self.neutron('router-list'))
        self.assertTableStruct(router_list, ['id', 'name',
                                             'external_gateway_info'])

    def test_neutron_security_group_list(self):
        security_grp = self.parser.listing(self.neutron('security-group-list'))
        self.assertTableStruct(security_grp, ['id', 'name',
                                              'security_group_rules'])

    def test_neutron_security_group_rule_list(self):
        security_grp = self.parser.listing(self.neutron
                                           ('security-group-rule-list'))
        self.assertTableStruct(security_grp, ['id', 'security_group',
                                              'direction', 'ethertype',
                                              'port/protocol', 'remote'])

    def test_neutron_subnet_list(self):
        subnet_list = self.parser.listing(self.neutron('subnet-list'))
        self.assertTableStruct(subnet_list, ['id', 'name', 'cidr',
                                             'allocation_pools'])

    def test_neutron_help(self):
        help_text = self.neutron('help')
        lines = help_text.split('\n')
        self.assertFirstLineStartsWith(lines, 'usage: neutron')

        commands = []
        cmds_start = lines.index('Commands for API v2.0:')
        command_pattern = re.compile(r'^ {2}([a-z0-9\-\_]+)')
        for line in lines[cmds_start:]:
            match = command_pattern.match(line)
            if match:
                commands.append(match.group(1))
        commands = set(commands)
        wanted_commands = set(('net-create', 'subnet-list', 'port-delete',
                               'router-show', 'agent-update', 'help'))
        self.assertFalse(wanted_commands - commands)

    # Optional arguments:

    def test_neutron_version(self):
        self.neutron('', flags='--version')

    def test_neutron_debug_net_list(self):
        self.neutron('net-list', flags='--debug')

    def test_neutron_quiet_net_list(self):
        self.neutron('net-list', flags='--quiet')



# Copyright 2016 Comcast Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.qos import dscp_marking_rule as dscp_rule
from neutronclient.tests.unit import test_cli20


class CLITestV20QoSDscpMarkingRuleJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['dscp_marking_rule']

    def setUp(self):
        super(CLITestV20QoSDscpMarkingRuleJSON, self).setUp()
        self.dscp_res = 'dscp_marking_rule'
        self.dscp_cmd_res = 'qos_dscp_marking_rule'
        self.dscp_ress = self.dscp_res + 's'
        self.dscp_cmd_ress = self.dscp_cmd_res + 's'

    def test_create_dscp_marking_rule_with_dscp_mark(self):
        cmd = dscp_rule.CreateQoSDscpMarkingRule(test_cli20.MyApp(sys.stdout),
                                                 None)
        my_id = 'my-id'
        policy_id = 'policy_id'
        position_names = ['dscp_mark']
        valid_dscp_marks = ['0', '56']
        invalid_dscp_marks = ['-1', '19', '42', '44', '57', '58']
        for dscp_mark in valid_dscp_marks:
            args = ['--dscp-mark', dscp_mark, policy_id]
            position_values = [dscp_mark]
            self._test_create_resource(self.dscp_res, cmd, '', my_id, args,
                                       position_names, position_values,
                                       cmd_resource=self.dscp_cmd_res,
                                       parent_id=policy_id)
        for dscp_mark in invalid_dscp_marks:
            args = ['--dscp-mark', dscp_mark, policy_id]
            position_values = [dscp_mark]
            self._test_create_resource(
                self.dscp_res, cmd, '', my_id, args,
                position_names, position_values,
                cmd_resource=self.dscp_cmd_res,
                parent_id=policy_id,
                no_api_call=True,
                expected_exception=exceptions.CommandError)

    def test_update_dscp_marking_rule_with_dscp_mark(self):
        cmd = dscp_rule.UpdateQoSDscpMarkingRule(test_cli20.MyApp(sys.stdout),
                                                 None)
        my_id = 'my-id'
        dscp_mark = '56'
        policy_id = 'policy_id'
        args = ['--dscp-mark', dscp_mark,
                my_id, policy_id]
        self._test_update_resource(self.dscp_res, cmd, my_id, args,
                                   {'dscp_mark': dscp_mark},
                                   cmd_resource=self.dscp_cmd_res,
                                   parent_id=policy_id)

    def test_delete_dscp_marking_rule(self):
        cmd = dscp_rule.DeleteQoSDscpMarkingRule(test_cli20.MyApp(sys.stdout),
                                                 None)
        my_id = 'my-id'
        policy_id = 'policy_id'
        args = [my_id, policy_id]
        self._test_delete_resource(self.dscp_res, cmd, my_id, args,
                                   cmd_resource=self.dscp_cmd_res,
                                   parent_id=policy_id)

    def test_show_dscp_marking_rule(self):
        cmd = dscp_rule.ShowQoSDscpMarkingRule(test_cli20.MyApp(sys.stdout),
                                               None)
        policy_id = 'policy_id'
        args = ['--fields', 'id', self.test_id, policy_id]
        self._test_show_resource(self.dscp_res, cmd, self.test_id, args,
                                 ['id'], cmd_resource=self.dscp_cmd_res,
                                 parent_id=policy_id)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from oslo_utils import uuidutils
import testtools

from neutronclient.common import exceptions
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.tests.unit import test_cli20
from neutronclient.v2_0 import client


class CLITestNameorID(testtools.TestCase):

    def setUp(self):
        """Prepare the test environment."""
        super(CLITestNameorID, self).setUp()
        self.endurl = test_cli20.ENDURL
        self.client = client.Client(token=test_cli20.TOKEN,
                                    endpoint_url=self.endurl)

    def test_get_id_from_id(self):
        _id = uuidutils.generate_uuid()
        reses = {'networks': [{'id': _id, }, ], }
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "networks_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            returned_id = neutronV20.find_resourceid_by_name_or_id(
                self.client, 'network', _id)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&id=" + _id),
                self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertEqual(_id, returned_id)

    def test_get_id_from_id_then_name_empty(self):
        _id = uuidutils.generate_uuid()
        reses = {'networks': [{'id': _id, }, ], }
        resstr = self.client.serialize(reses)
        resstr1 = self.client.serialize({'networks': []})
        path = getattr(self.client, "networks_path")
        with mock.patch.object(self.client.httpclient,
                               "request") as mock_request:
            mock_request.side_effect = [(test_cli20.MyResp(200), resstr1),
                                        (test_cli20.MyResp(200), resstr)]
            returned_id = neutronV20.find_resourceid_by_name_or_id(
                self.client, 'network', _id)

        self.assertEqual(2, mock_request.call_count)
        mock_request.assert_has_calls([
            mock.call(
                test_cli20.MyUrlComparator(
                    test_cli20.end_url(path, "fields=id&id=" + _id),
                    self.client),
                'GET',
                body=None,
                headers=test_cli20.ContainsKeyValue(
                    {'X-Auth-Token': test_cli20.TOKEN})),
            mock.call(
                test_cli20.MyUrlComparator(
                    test_cli20.end_url(path, "fields=id&name=" + _id),
                    self.client),
                'GET',
                body=None,
                headers=test_cli20.ContainsKeyValue(
                    {'X-Auth-Token': test_cli20.TOKEN}))])
        self.assertEqual(_id, returned_id)

    def test_get_id_from_name(self):
        name = 'myname'
        _id = uuidutils.generate_uuid()
        reses = {'networks': [{'id': _id, }, ], }
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "networks_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            returned_id = neutronV20.find_resourceid_by_name_or_id(
                self.client, 'network', name)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&name=" + name),
                self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertEqual(_id, returned_id)

    def test_get_id_from_name_multiple(self):
        name = 'myname'
        reses = {'networks': [{'id': uuidutils.generate_uuid()},
                              {'id': uuidutils.generate_uuid()}]}
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "networks_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            exception = self.assertRaises(
                exceptions.NeutronClientNoUniqueMatch,
                neutronV20.find_resourceid_by_name_or_id,
                self.client, 'network', name)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&name=" + name),
                self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertIn('Multiple', exception.message)

    def test_get_id_from_name_notfound(self):
        name = 'myname'
        reses = {'networks': []}
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "networks_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            exception = self.assertRaises(
                exceptions.NotFound,
                neutronV20.find_resourceid_by_name_or_id,
                self.client, 'network', name)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&name=" + name),
                self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertIn('Unable to find', exception.message)
        self.assertEqual(404, exception.status_code)

    def test_get_id_from_name_multiple_with_project(self):
        name = 'web_server'
        project = uuidutils.generate_uuid()
        expect_id = uuidutils.generate_uuid()
        reses = {'security_groups':
                 [{'id': expect_id, 'tenant_id': project}]}
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "security_groups_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            observed_id = neutronV20.find_resourceid_by_name_or_id(
                self.client, 'security_group', name, project)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&name=%s&tenant_id=%s" %
                                         (name, project)), self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertEqual(expect_id, observed_id)

    def test_get_id_from_name_multiple_with_project_not_found(self):
        name = 'web_server'
        project = uuidutils.generate_uuid()
        resstr_notfound = self.client.serialize({'security_groups': []})
        resp = (test_cli20.MyResp(200), resstr_notfound)
        path = getattr(self.client, "security_groups_path")
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            exc = self.assertRaises(exceptions.NotFound,
                                    neutronV20.find_resourceid_by_name_or_id,
                                    self.client, 'security_group', name,
                                    project)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, "fields=id&name=%s&tenant_id=%s" %
                                         (name, project)), self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        self.assertIn('Unable to find', exc.message)
        self.assertEqual(404, exc.status_code)

    def _test_get_resource_by_id(self, id_only=False):
        _id = uuidutils.generate_uuid()
        net = {'id': _id, 'name': 'test'}
        reses = {'networks': [net], }
        resstr = self.client.serialize(reses)
        resp = (test_cli20.MyResp(200), resstr)
        path = getattr(self.client, "networks_path")
        if id_only:
            query_params = "fields=id&id=%s" % _id
        else:
            query_params = "id=%s" % _id
        with mock.patch.object(self.client.httpclient, "request",
                               return_value=resp) as mock_request:
            if id_only:
                returned_id = neutronV20.find_resourceid_by_id(
                    self.client, 'network', _id)
                self.assertEqual(_id, returned_id)
            else:
                result = neutronV20.find_resource_by_id(
                    self.client, 'network', _id)
                self.assertEqual(net, result)

        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path, query_params),
                self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def test_get_resource_by_id(self):
        self._test_get_resource_by_id(id_only=False)

    def test_get_resourceid_by_id(self):
        self._test_get_resource_by_id(id_only=True)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


class ListFlavorProfile(neutronV20.ListCommand):
    """List Neutron service flavor profiles."""

    resource = 'service_profile'
    list_columns = ['id', 'description', 'enabled', 'metainfo']
    pagination_support = True
    sorting_support = True


class ShowFlavorProfile(neutronV20.ShowCommand):
    """Show information about a given Neutron service flavor profile."""

    resource = 'service_profile'


class CreateFlavorProfile(neutronV20.CreateCommand):
    """Create a Neutron service flavor profile."""

    resource = 'service_profile'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description for the flavor profile.'))
        parser.add_argument(
            '--driver',
            help=_('Python module path to driver.'))
        parser.add_argument(
            '--metainfo',
            help=_('Metainfo for the flavor profile.'))
        utils.add_boolean_argument(
            parser,
            '--enabled',
            default=argparse.SUPPRESS,
            help=_('Sets enabled flag.'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body,
                               ['description', 'driver', 'enabled',
                                'metainfo'])
        return {self.resource: body}


class DeleteFlavorProfile(neutronV20.DeleteCommand):
    """Delete a given Neutron service flavor profile."""

    resource = 'service_profile'


class UpdateFlavorProfile(neutronV20.UpdateCommand):
    """Update a given Neutron service flavor profile."""

    resource = 'service_profile'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--description',
            help=_('Description for the flavor profile.'))
        parser.add_argument(
            '--driver',
            help=_('Python module path to driver.'))
        parser.add_argument(
            '--metainfo',
            help=_('Metainfo for the flavor profile.'))
        utils.add_boolean_argument(
            parser,
            '--enabled',
            default=argparse.SUPPRESS,
            help=_('Sets enabled flag.'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body,
                               ['description', 'driver', 'enabled',
                                'metainfo'])
        return {self.resource: body}



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _

LOG = logging.getLogger(__name__)

resource = 'port_pair'

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('ingress', 'Ingress Logical Port', column_util.LIST_BOTH),
    ('egress', 'Egress Logical Port', column_util.LIST_BOTH),
    ('service_function_parameters', 'Service Function Parameters',
     column_util.LIST_LONG_ONLY),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('project_id', 'Project', column_util.LIST_LONG_ONLY),
)


class CreateSfcPortPair(command.ShowOne):
    _description = _("Create a port pair")

    def get_parser(self, prog_name):
        parser = super(CreateSfcPortPair, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the port pair'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the port pair'))
        parser.add_argument(
            '--service-function-parameters',
            metavar='correlation=<correlation-type>,weight=<weight>',
            action=parseractions.MultiKeyValueAction,
            optional_keys=['correlation', 'weight'],
            help=_('Dictionary of service function parameters. '
                   'Currently, correlation=(None|mpls|nsh) and weight '
                   'are supported. Weight is an integer that influences '
                   'the selection of a port pair within a port pair group '
                   'for a flow. The higher the weight, the more flows will '
                   'hash to the port pair. The default weight is 1.'))
        parser.add_argument(
            '--ingress',
            metavar='<ingress>',
            required=True,
            help=_('Ingress neutron port (name or ID)'))
        parser.add_argument(
            '--egress',
            metavar='<egress>',
            required=True,
            help=_('Egress neutron port (name or ID)'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        body = {resource: attrs}
        obj = client.create_sfc_port_pair(body)[resource]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteSfcPortPair(command.Command):
    _description = _("Delete a given port pair")

    def get_parser(self, prog_name):
        parser = super(DeleteSfcPortPair, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair',
            metavar="<port-pair>",
            help=_("Port pair to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        # TODO(mohan): Add support for deleting multiple resources.
        client = self.app.client_manager.neutronclient
        port_pair_id = _get_id(client, parsed_args.port_pair, resource)
        try:
            client.delete_sfc_port_pair(port_pair_id)
        except Exception as e:
            msg = (_("Failed to delete port pair with name "
                     "or ID '%(port_pair)s': %(e)s")
                   % {'port_pair': parsed_args.port_pair, 'e': e})
            raise exceptions.CommandError(msg)


class ListSfcPortPair(command.Lister):
    _description = _("List port pairs")

    def get_parser(self, prog_name):
        parser = super(ListSfcPortPair, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        data = client.list_sfc_port_pairs()
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data['port_pairs']))


class SetSfcPortPair(command.Command):
    _description = _("Set port pair properties")

    def get_parser(self, prog_name):
        parser = super(SetSfcPortPair, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the port pair'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the port pair'))
        parser.add_argument(
            'port_pair',
            metavar='<port-pair>',
            help=_("Port pair to modify (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        port_pair_id = _get_id(client, parsed_args.port_pair, resource)
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        body = {resource: attrs}
        try:
            client.update_sfc_port_pair(port_pair_id, body)
        except Exception as e:
            msg = (_("Failed to update port pair '%(port_pair)s': %(e)s")
                   % {'port_pair': parsed_args.port_pair, 'e': e})
            raise exceptions.CommandError(msg)


class ShowSfcPortPair(command.ShowOne):
    _description = _("Display port pair details")

    def get_parser(self, prog_name):
        parser = super(ShowSfcPortPair, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair',
            metavar='<port-pair>',
            help=_("Port pair to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        port_pair_id = _get_id(client, parsed_args.port_pair, resource)
        obj = client.show_sfc_port_pair(port_pair_id)[resource]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = parsed_args.name
    if parsed_args.description is not None:
        attrs['description'] = parsed_args.description
    if is_create:
        _get_attrs(client_manager, attrs, parsed_args)
    return attrs


def _get_attrs(client_manager, attrs, parsed_args):
    if parsed_args.ingress is not None:
        attrs['ingress'] = _get_id(client_manager.neutronclient,
                                   parsed_args.ingress, 'port')
    if parsed_args.egress is not None:
        attrs['egress'] = _get_id(client_manager.neutronclient,
                                  parsed_args.egress, 'port')
    if parsed_args.service_function_parameters is not None:
        attrs['service_function_parameters'] = _get_service_function_params(
            parsed_args.service_function_parameters)


def _get_service_function_params(sf_params):
    attrs = {}
    for sf_param in sf_params:
        if 'correlation' in sf_param:
            if sf_param['correlation'] == 'None':
                attrs['correlation'] = None
            else:
                attrs['correlation'] = sf_param['correlation']
        if 'weight' in sf_param:
            attrs['weight'] = sf_param['weight']
    return attrs


def _get_id(client, id_or_name, resource):
    return client.find_resource(resource, id_or_name)['id']



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0 import servicetype
from neutronclient.tests.unit import test_cli20


class CLITestV20ServiceProvidersJSON(test_cli20.CLITestV20Base):
    id_field = "name"

    def setUp(self):
        super(CLITestV20ServiceProvidersJSON, self).setUp(
            plurals={'tags': 'tag'}
        )

    def test_list_service_providers(self):
        resources = "service_providers"
        cmd = servicetype.ListServiceProvider(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, True)

    def test_list_service_providers_pagination(self):
        resources = "service_providers"
        cmd = servicetype.ListServiceProvider(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_service_providers_sort(self):
        resources = "service_providers"
        cmd = servicetype.ListServiceProvider(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name"],
                                  sort_dir=["asc", "desc"])

    def test_list_service_providers_limit(self):
        resources = "service_providers"
        cmd = servicetype.ListServiceProvider(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_list_resources(resources, cmd, page_size=1000)



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import testtools

from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron.v2_0.vpn import utils as vpn_utils


class TestVPNUtils(testtools.TestCase):

    def test_validate_lifetime_dictionary_seconds(self):
        input_str = utils.str2dict("units=seconds,value=3600")
        self.assertIsNone(vpn_utils.validate_lifetime_dict(input_str))

    def test_validate_dpd_dictionary_action_hold(self):
        input_str = utils.str2dict("action=hold,interval=30,timeout=120")
        self.assertIsNone(vpn_utils.validate_dpd_dict(input_str))

    def test_validate_dpd_dictionary_action_restart(self):
        input_str = utils.str2dict("action=restart,interval=30,timeout=120")
        self.assertIsNone(vpn_utils.validate_dpd_dict(input_str))

    def test_validate_dpd_dictionary_action_restart_by_peer(self):
        input_str = utils.str2dict(
            "action=restart-by-peer,interval=30,timeout=120"
        )
        self.assertIsNone(vpn_utils.validate_dpd_dict(input_str))

    def test_validate_dpd_dictionary_action_clear(self):
        input_str = utils.str2dict('action=clear,interval=30,timeout=120')
        self.assertIsNone(vpn_utils.validate_dpd_dict(input_str))

    def test_validate_dpd_dictionary_action_disabled(self):
        input_str = utils.str2dict('action=disabled,interval=30,timeout=120')
        self.assertIsNone(vpn_utils.validate_dpd_dict(input_str))

    def test_validate_lifetime_dictionary_invalid_unit_key(self):
        input_str = utils.str2dict('ut=seconds,value=3600')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_invalid_unit_key_value(self):
        input_str = utils.str2dict('units=seconds,val=3600')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_unsupported_units(self):
        input_str = utils.str2dict('units=minutes,value=3600')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_invalid_empty_unit(self):
        input_str = utils.str2dict('units=,value=3600')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_under_minimum_integer_value(self):
        input_str = utils.str2dict('units=seconds,value=59')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_negative_integer_value(self):
        input_str = utils.str2dict('units=seconds,value=-1')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_lifetime_dictionary_empty_value(self):
        input_str = utils.str2dict('units=seconds,value=')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_dpd_dictionary_invalid_key_action(self):
        input_str = utils.str2dict('act=hold,interval=30,timeout=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_invalid_key_interval(self):
        input_str = utils.str2dict('action=hold,int=30,timeout=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_invalid_key_timeout(self):
        input_str = utils.str2dict('action=hold,interval=30,tiut=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_unsupported_action(self):
        input_str = utils.str2dict('action=bye-bye,interval=30,timeout=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_empty_action(self):
        input_str = utils.str2dict('action=,interval=30,timeout=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_empty_interval(self):
        input_str = utils.str2dict('action=hold,interval=,timeout=120')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_negative_interval_value(self):
        input_str = utils.str2dict('action=hold,interval=-1,timeout=120')
        self._test_validate_lifetime_negative_test_case(input_str)

    def test_validate_dpd_dictionary_zero_timeout(self):
        input_str = utils.str2dict('action=hold,interval=30,timeout=0')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_empty_timeout(self):
        input_str = utils.str2dict('action=hold,interval=30,timeout=')
        self._test_validate_dpd_negative_test_case(input_str)

    def test_validate_dpd_dictionary_negative_timeout_value(self):
        input_str = utils.str2dict('action=hold,interval=30,timeout=-1')
        self._test_validate_lifetime_negative_test_case(input_str)

    def _test_validate_lifetime_negative_test_case(self, input_str):
        """Generic handler for negative lifetime tests."""
        self.assertRaises(exceptions.CommandError,
                          vpn_utils.validate_lifetime_dict,
                          (input_str))

    def _test_validate_dpd_negative_test_case(self, input_str):
        """Generic handler for negative lifetime tests."""
        self.assertRaises(exceptions.CommandError,
                          vpn_utils.validate_lifetime_dict,
                          (input_str))



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient.neutron.v2_0 import extension
from neutronclient.tests.unit import test_cli20


class CLITestV20Extension(test_cli20.CLITestV20Base):
    id_field = 'alias'

    def test_list_extensions(self):
        resources = 'extensions'
        cmd = extension.ListExt(test_cli20.MyApp(sys.stdout), None)
        contents = [{'alias': 'ext1', 'name': 'name1', 'other': 'other1'},
                    {'alias': 'ext2', 'name': 'name2', 'other': 'other2'}]
        ret = self._test_list_resources(resources, cmd,
                                        response_contents=contents)
        ret_words = set(ret.split())
        # Check only the default columns are shown.
        self.assertIn('name', ret_words)
        self.assertIn('alias', ret_words)
        self.assertNotIn('other', ret_words)

    def test_show_extension(self):
        # -F option does not work for ext-show at the moment, so -F option
        # is not passed in the commandline args as other tests do.
        resource = 'extension'
        cmd = extension.ShowExt(test_cli20.MyApp(sys.stdout), None)
        args = [self.test_id]
        ext_alias = self.test_id
        self._test_show_resource(resource, cmd, ext_alias, args, fields=[])



#    (c) Copyright 2015 Cisco Systems, Inc.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.vpn import endpoint_group
from neutronclient.tests.unit import test_cli20


class CLITestV20VpnEndpointGroupJSON(test_cli20.CLITestV20Base):

    def setUp(self):
        super(CLITestV20VpnEndpointGroupJSON, self).setUp()
        self.register_non_admin_status_resource('endpoint_group')

    def test_create_endpoint_group_with_cidrs(self):
        # vpn-endpoint-group-create with CIDR endpoints."""
        resource = 'endpoint_group'
        cmd = endpoint_group.CreateEndpointGroup(test_cli20.MyApp(sys.stdout),
                                                 None)
        tenant_id = 'mytenant-id'
        my_id = 'my-id'
        name = 'my-endpoint-group'
        description = 'my endpoint group'
        endpoint_type = 'cidr'
        endpoints = ['10.0.0.0/24', '20.0.0.0/24']

        args = ['--name', name,
                '--description', description,
                '--tenant-id', tenant_id,
                '--type', endpoint_type,
                '--value', '10.0.0.0/24',
                '--value', '20.0.0.0/24']

        position_names = ['name', 'description', 'tenant_id',
                          'type', 'endpoints']

        position_values = [name, description, tenant_id,
                           endpoint_type, endpoints]

        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_endpoint_group_with_subnets(self):
        # vpn-endpoint-group-create with subnet endpoints."""
        resource = 'endpoint_group'
        cmd = endpoint_group.CreateEndpointGroup(test_cli20.MyApp(sys.stdout),
                                                 None)
        tenant_id = 'mytenant-id'
        my_id = 'my-id'
        endpoint_type = 'subnet'
        subnet = 'subnet-id'
        endpoints = [subnet]

        args = ['--type', endpoint_type,
                '--value', subnet,
                '--tenant-id', tenant_id]

        position_names = ['type', 'endpoints', 'tenant_id']

        position_values = [endpoint_type, endpoints, tenant_id]

        self._test_create_resource(resource, cmd, None, my_id, args,
                                   position_names, position_values)

    def test_list_endpoint_group(self):
        # vpn-endpoint-group-list.
        resources = "endpoint_groups"
        cmd = endpoint_group.ListEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        self._test_list_resources(resources, cmd, True)

    def test_list_endpoint_group_pagination(self):
        # vpn-endpoint-group-list.
        resources = "endpoint_groups"
        cmd = endpoint_group.ListEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_endpoint_group_sort(self):
        # vpn-endpoint-group-list --sort-key name --sort-key id
        # --sort-key asc --sort-key desc
        resources = "endpoint_groups"
        cmd = endpoint_group.ListEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_endpoint_group_limit(self):
        # vpn-endpoint-group-list -P.
        resources = "endpoint_groups"
        cmd = endpoint_group.ListEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_endpoint_group_id(self):
        # vpn-endpoint-group-show test_id.
        resource = 'endpoint_group'
        cmd = endpoint_group.ShowEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_endpoint_group_id_name(self):
        # vpn-endpoint-group-show.
        resource = 'endpoint_group'
        cmd = endpoint_group.ShowEndpointGroup(test_cli20.MyApp(sys.stdout),
                                               None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_endpoint_group(self):
        # vpn-endpoint-group-update myid --name newname --description newdesc.
        resource = 'endpoint_group'
        cmd = endpoint_group.UpdateEndpointGroup(test_cli20.MyApp(sys.stdout),
                                                 None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname',
                                    '--description', 'newdesc'],
                                   {'name': 'newname',
                                    'description': 'newdesc'})

    def test_delete_endpoint_group(self):
        # vpn-endpoint-group-delete my-id.
        resource = 'endpoint_group'
        cmd = endpoint_group.DeleteEndpointGroup(test_cli20.MyApp(sys.stdout),
                                                 None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



# Copyright 2015 Huawei Technologies India Pvt Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

import testscenarios

from neutronclient.neutron.v2_0 import rbac
from neutronclient.tests.unit import test_cli20

load_tests = testscenarios.load_tests_apply_scenarios


class CLITestV20RBACBaseJSON(test_cli20.CLITestV20Base):
    non_admin_status_resources = ['rbac_policy']

    scenarios = [
        ('network rbac objects',
         {'object_type_name': 'network', 'object_type_val': 'network'}),
        ('qos policy rbac objects',
         {'object_type_name': 'qos-policy', 'object_type_val': 'qos_policy'}),
    ]

    def test_create_rbac_policy_with_mandatory_params(self):
        # Create rbac: rbac_object --type <object_type_name> --action
        # access_as_shared
        resource = 'rbac_policy'
        cmd = rbac.CreateRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        name = 'rbac_object'
        myid = 'myid'
        args = [name, '--type', self.object_type_name,
                '--action', 'access_as_shared']
        position_names = ['object_id', 'object_type',
                          'target_tenant', 'action']
        position_values = [name, self.object_type_val, '*',
                           'access_as_shared']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_rbac_policy_with_all_params(self):
        # Create rbac: rbac_object --type <object_type_name>
        # --target-tenant tenant_id --action access_as_external
        resource = 'rbac_policy'
        cmd = rbac.CreateRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        name = 'rbac_object'
        myid = 'myid'
        args = [name, '--type', self.object_type_name,
                '--target-tenant', 'tenant_id',
                '--action', 'access_as_external']
        position_names = ['object_id', 'object_type',
                          'target_tenant', 'action']
        position_values = [name, self.object_type_val, 'tenant_id',
                           'access_as_external']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_rbac_policy_with_unicode(self):
        # Create rbac policy u'\u7f51\u7edc'.
        resource = 'rbac_policy'
        cmd = rbac.CreateRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        name = u'\u7f51\u7edc'
        myid = 'myid'
        args = [name, '--type', self.object_type_name,
                '--target-tenant', 'tenant_id',
                '--action', 'access_as_external']
        position_names = ['object_id', 'object_type',
                          'target_tenant', 'action']
        position_values = [name, self.object_type_val, 'tenant_id',
                           'access_as_external']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_update_rbac_policy(self):
        # rbac-update <rbac-uuid> --target-tenant <other-tenant-uuid>.
        resource = 'rbac_policy'
        cmd = rbac.UpdateRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--target-tenant', 'tenant_id'],
                                   {'target_tenant': 'tenant_id', })

    def test_delete_rbac_policy(self):
        # rbac-delete my-id.
        resource = 'rbac_policy'
        cmd = rbac.DeleteRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'myid1'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_list_rbac_policies(self):
        # rbac-list.
        resources = "rbac_policies"
        cmd = rbac.ListRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_rbac_policies_pagination(self):
        # rbac-list with pagination.
        resources = "rbac_policies"
        cmd = rbac.ListRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_rbac_policies_sort(self):
        # sorted list:
        # rbac-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "rbac_policies"
        cmd = rbac.ListRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_rbac_policies_limit(self):
        # size (1000) limited list: rbac-list -P.
        resources = "rbac_policies"
        cmd = rbac.ListRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_rbac_policy(self):
        # rbac-show test_id.
        resource = 'rbac_policy'
        cmd = rbac.ShowRBACPolicy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])



# Copyright 2016 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.neutron.v2_0.bgp import speaker as bgp_speaker


def add_common_args(parser):
    parser.add_argument('dragent_id',
                        metavar='BGP_DRAGENT_ID',
                        help=_('ID of the Dynamic Routing agent.'))
    parser.add_argument('bgp_speaker',
                        metavar='BGP_SPEAKER',
                        help=_('ID or name of the BGP speaker.'))


class AddBGPSpeakerToDRAgent(neutronV20.NeutronCommand):
    """Add a BGP speaker to a Dynamic Routing agent."""

    def get_parser(self, prog_name):
        parser = super(AddBGPSpeakerToDRAgent, self).get_parser(prog_name)
        add_common_args(parser)
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = bgp_speaker.get_bgp_speaker_id(neutron_client,
                                                     parsed_args.bgp_speaker)
        neutron_client.add_bgp_speaker_to_dragent(
            parsed_args.dragent_id, {'bgp_speaker_id': _speaker_id})
        print(_('Associated BGP speaker %s to the Dynamic Routing agent.')
              % parsed_args.bgp_speaker, file=self.app.stdout)


class RemoveBGPSpeakerFromDRAgent(neutronV20.NeutronCommand):
    """Removes a BGP speaker from a Dynamic Routing agent."""

    def get_parser(self, prog_name):
        parser = super(RemoveBGPSpeakerFromDRAgent, self).get_parser(
            prog_name)
        add_common_args(parser)
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _speaker_id = bgp_speaker.get_bgp_speaker_id(neutron_client,
                                                     parsed_args.bgp_speaker)
        neutron_client.remove_bgp_speaker_from_dragent(parsed_args.dragent_id,
                                                       _speaker_id)
        print(_('Disassociated BGP speaker %s from the '
                'Dynamic Routing agent.')
              % parsed_args.bgp_speaker, file=self.app.stdout)


class ListBGPSpeakersOnDRAgent(neutronV20.ListCommand):
    """List BGP speakers hosted by a Dynamic Routing agent."""

    list_columns = ['id', 'name', 'local_as', 'ip_version']
    resource = 'bgp_speaker'

    def get_parser(self, prog_name):
        parser = super(ListBGPSpeakersOnDRAgent,
                       self).get_parser(prog_name)
        parser.add_argument(
            'dragent_id',
            metavar='BGP_DRAGENT_ID',
            help=_('ID of the Dynamic Routing agent.'))
        return parser

    def call_server(self, neutron_client, search_opts, parsed_args):
        data = neutron_client.list_bgp_speaker_on_dragent(
            parsed_args.dragent_id, **search_opts)
        return data


class ListDRAgentsHostingBGPSpeaker(neutronV20.ListCommand):
    """List Dynamic Routing agents hosting a BGP speaker."""

    resource = 'agent'
    _formatters = {}
    list_columns = ['id', 'host', 'admin_state_up', 'alive']
    unknown_parts_flag = False

    def get_parser(self, prog_name):
        parser = super(ListDRAgentsHostingBGPSpeaker,
                       self).get_parser(prog_name)
        parser.add_argument('bgp_speaker',
                            metavar='BGP_SPEAKER',
                            help=_('ID or name of the BGP speaker.'))
        return parser

    def extend_list(self, data, parsed_args):
        for agent in data:
            agent['alive'] = ":-)" if agent['alive'] else 'xxx'

    def call_server(self, neutron_client, search_opts, parsed_args):
        _speaker_id = bgp_speaker.get_bgp_speaker_id(neutron_client,
                                                     parsed_args.bgp_speaker)
        search_opts['bgp_speaker'] = _speaker_id
        data = neutron_client.list_dragents_hosting_bgp_speaker(**search_opts)
        return data



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.neutron.v2_0 import agentscheduler
from neutronclient.neutron.v2_0 import network
from neutronclient.tests.unit import test_cli20


AGENT_ID = 'agent_id1'
NETWORK_ID = 'net_id1'
ROUTER_ID = 'router_id1'


class CLITestV20AgentScheduler(test_cli20.CLITestV20Base):
    def _test_add_to_agent(self, resource, cmd, cmd_args, destination,
                           body, result):
        path = ((self.client.agent_path + destination) %
                cmd_args[0])

        result_str = self.client.serialize(result)
        return_tup = (test_cli20.MyResp(200), result_str)

        cmd_parser = cmd.get_parser('test_' + resource)
        parsed_args = cmd_parser.parse_args(cmd_args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)
        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path), 'POST',
            body=test_cli20.MyComparator(body, self.client),
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def _test_remove_from_agent(self, resource, cmd, cmd_args, destination):
        path = ((self.client.agent_path + destination + '/%s') %
                cmd_args)

        return_tup = (test_cli20.MyResp(204), None)
        cmd_parser = cmd.get_parser('test_' + resource)
        parsed_args = cmd_parser.parse_args(cmd_args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)
        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.end_url(path), 'DELETE',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))


class CLITestV20DHCPAgentScheduler(CLITestV20AgentScheduler):

    def test_add_network_to_agent(self):
        resource = 'agent'
        cmd = agentscheduler.AddNetworkToDhcpAgent(
            test_cli20.MyApp(sys.stdout), None)
        args = (AGENT_ID, NETWORK_ID)
        body = {'network_id': NETWORK_ID}
        result = {'network_id': 'net_id', }
        self._test_add_to_agent(resource, cmd, args, self.client.DHCP_NETS,
                                body, result)

    def test_remove_network_from_agent(self):
        resource = 'agent'
        cmd = agentscheduler.RemoveNetworkFromDhcpAgent(
            test_cli20.MyApp(sys.stdout), None)
        args = (AGENT_ID, NETWORK_ID)
        self._test_remove_from_agent(resource, cmd, args,
                                     self.client.DHCP_NETS)

    @mock.patch.object(network.ListNetwork, "extend_list")
    def test_list_networks_on_agent(self, mock_extend_list):
        resources = 'networks'
        cmd = agentscheduler.ListNetworksOnDhcpAgent(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.agent_path + self.client.DHCP_NETS) %
                agent_id)
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def test_list_agents_hosting_network(self):
        resources = 'agent'
        cmd = agentscheduler.ListDhcpAgentsHostingNetwork(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.network_path + self.client.DHCP_AGENTS) %
                agent_id)
        contents = {self.id_field: 'myid1', 'alive': True}
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path, response_contents=contents)


class CLITestV20L3AgentScheduler(CLITestV20AgentScheduler):

    def test_add_router_to_agent(self):
        resource = 'agent'
        cmd = agentscheduler.AddRouterToL3Agent(
            test_cli20.MyApp(sys.stdout), None)
        args = (AGENT_ID, ROUTER_ID)
        body = {'router_id': ROUTER_ID}
        result = {'network_id': 'net_id', }
        self._test_add_to_agent(resource, cmd, args, self.client.L3_ROUTERS,
                                body, result)

    def test_remove_router_from_agent(self):
        resource = 'agent'
        cmd = agentscheduler.RemoveRouterFromL3Agent(
            test_cli20.MyApp(sys.stdout), None)
        args = (AGENT_ID, ROUTER_ID)
        self._test_remove_from_agent(resource, cmd, args,
                                     self.client.L3_ROUTERS)

    def test_list_routers_on_agent(self):
        resources = 'router'
        cmd = agentscheduler.ListRoutersOnL3Agent(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.agent_path + self.client.L3_ROUTERS) %
                agent_id)
        contents = {self.id_field: 'myid1', 'name': 'my_name'}
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path, response_contents=contents)

    def test_list_agents_hosting_router(self):
        resources = 'agent'
        cmd = agentscheduler.ListL3AgentsHostingRouter(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.router_path + self.client.L3_AGENTS) %
                agent_id)
        contents = {self.id_field: 'myid1', 'alive': True}
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path, response_contents=contents)


class CLITestV20LBaaSAgentScheduler(test_cli20.CLITestV20Base):

    def test_list_pools_on_agent(self):
        resources = 'pools'
        cmd = agentscheduler.ListPoolsOnLbaasAgent(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.agent_path + self.client.LOADBALANCER_POOLS) %
                agent_id)
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path)

    def test_get_lbaas_agent_hosting_pool(self):
        resources = 'agent'
        cmd = agentscheduler.GetLbaasAgentHostingPool(
            test_cli20.MyApp(sys.stdout), None)
        pool_id = 'pool_id1'
        path = ((self.client.pool_path + self.client.LOADBALANCER_AGENT) %
                pool_id)
        contents = {self.id_field: 'myid1', 'alive': True}
        self._test_list_resources(resources, cmd, base_args=[pool_id],
                                  path=path, response_contents=contents)


class CLITestV20LBaaSV2AgentScheduler(test_cli20.CLITestV20Base):

    def test_list_loadbalancers_on_agent(self):
        resources = 'loadbalancers'
        cmd = agentscheduler.ListLoadBalancersOnLbaasAgent(
            test_cli20.MyApp(sys.stdout), None)
        agent_id = 'agent_id1'
        path = ((self.client.agent_path + self.client.AGENT_LOADBALANCERS) %
                agent_id)
        self._test_list_resources(resources, cmd, base_args=[agent_id],
                                  path=path)

    def test_get_lbaas_agent_hosting_pool(self):
        resources = 'agent'
        cmd = agentscheduler.GetLbaasAgentHostingLoadBalancer(
            test_cli20.MyApp(sys.stdout), None)
        lb_id = 'lb_id1'
        path = ((self.client.lbaas_loadbalancer_path +
                 self.client.LOADBALANCER_HOSTING_AGENT) % lb_id)
        contents = {self.id_field: 'myid1', 'alive': True}
        self._test_list_resources(resources, cmd, base_args=[lb_id],
                                  path=path, response_contents=contents)



# Copyright (c) 2019 SUSE Linux Products GmbH
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Subnet onboard action implementation"""
import logging

from osc_lib.command import command
from osc_lib import exceptions

from neutronclient._i18n import _

LOG = logging.getLogger(__name__)


class NetworkOnboardSubnets(command.Command):
    """Onboard network subnets into a subnet pool"""

    def get_parser(self, prog_name):
        parser = super(NetworkOnboardSubnets, self).get_parser(prog_name)
        parser.add_argument(
            'network',
            metavar="<network>",
            help=_("Onboard all subnets associated with this network")
        )
        parser.add_argument(
            'subnetpool',
            metavar="<subnetpool>",
            help=_("Target subnet pool for onboarding subnets")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        subnetpool_id = _get_id(client, parsed_args.subnetpool, 'subnetpool')
        network_id = _get_id(client, parsed_args.network, 'network')
        body = {'network_id': network_id}
        try:
            client.onboard_network_subnets(subnetpool_id, body)
        except Exception as e:
            msg = (_("Failed to onboard subnets for network '%(n)s': %(e)s")
                   % {'n': parsed_args.network, 'e': e})
            raise exceptions.CommandError(msg)


def _get_id(client, id_or_name, resource):
    return client.find_resource(resource, str(id_or_name))['id']



# Copyright 2015 Huawei Technologies India Pvt Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.qos import bandwidth_limit_rule as bw_rule
from neutronclient.tests.unit import test_cli20


class CLITestV20QoSBandwidthLimitRuleJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['bandwidth_limit_rule']

    def setUp(self):
        super(CLITestV20QoSBandwidthLimitRuleJSON, self).setUp()
        self.res = 'bandwidth_limit_rule'
        self.cmd_res = 'qos_bandwidth_limit_rule'
        self.ress = self.res + 's'
        self.cmd_ress = self.cmd_res + 's'

    def test_create_bandwidth_limit_rule_with_max_kbps(self):
        cmd = bw_rule.CreateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-kbps', max_kbps, policy_id]
        position_names = ['max_kbps']
        position_values = [max_kbps]
        self._test_create_resource(self.res, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_create_bandwidth_limit_rule_with_max_burst_kbps(self):
        cmd = bw_rule.CreateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_burst_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-burst-kbps', max_burst_kbps, policy_id]
        position_names = ['max_burst_kbps']
        position_values = [max_burst_kbps]
        self._test_create_resource(self.res, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_create_bandwidth_limit_rule_with_all_params(self):
        cmd = bw_rule.CreateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_kbps = '1337'
        max_burst_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-kbps', max_kbps,
                '--max-burst-kbps', max_burst_kbps,
                policy_id]
        position_names = ['max_kbps', 'max_burst_kbps']
        position_values = [max_kbps, max_burst_kbps]
        self._test_create_resource(self.res, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_update_bandwidth_limit_rule_with_max_kbps(self):
        cmd = bw_rule.UpdateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-kbps', max_kbps, my_id, policy_id]
        self._test_update_resource(self.res, cmd, my_id, args,
                                   {'max_kbps': max_kbps, },
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_update_bandwidth_limit_rule_with_max_burst_kbps(self):
        cmd = bw_rule.UpdateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_burst_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-burst-kbps', max_burst_kbps,
                my_id, policy_id]
        self._test_update_resource(self.res, cmd, my_id, args,
                                   {'max_burst_kbps': max_burst_kbps},
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_update_bandwidth_limit_rule_with_all_params(self):
        cmd = bw_rule.UpdateQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        max_kbps = '1337'
        max_burst_kbps = '1337'
        policy_id = 'policy_id'
        args = ['--max-kbps', max_kbps,
                '--max-burst-kbps', max_burst_kbps,
                my_id, policy_id]
        self._test_update_resource(self.res, cmd, my_id, args,
                                   {'max_kbps': max_kbps,
                                    'max_burst_kbps': max_burst_kbps},
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_delete_bandwidth_limit_rule(self):
        cmd = bw_rule.DeleteQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'my-id'
        policy_id = 'policy_id'
        args = [my_id, policy_id]
        self._test_delete_resource(self.res, cmd, my_id, args,
                                   cmd_resource=self.cmd_res,
                                   parent_id=policy_id)

    def test_show_bandwidth_limit_rule(self):
        cmd = bw_rule.ShowQoSBandwidthLimitRule(test_cli20.MyApp(sys.stdout),
                                                None)
        policy_id = 'policy_id'
        args = ['--fields', 'id', self.test_id, policy_id]
        self._test_show_resource(self.res, cmd, self.test_id, args,
                                 ['id'], cmd_resource=self.cmd_res,
                                 parent_id=policy_id)



# Copyright 2015 Huawei Technologies India Pvt. Ltd..
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


class ListAddressScope(neutronV20.ListCommand):
    """List address scopes that belong to a given tenant."""

    resource = 'address_scope'
    list_columns = ['id', 'name', 'ip_version']
    pagination_support = True
    sorting_support = True


class ShowAddressScope(neutronV20.ShowCommand):
    """Show information about an address scope."""

    resource = 'address_scope'


class CreateAddressScope(neutronV20.CreateCommand):
    """Create an address scope for a given tenant."""

    resource = 'address_scope'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Set the address scope as shared.'))
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Specify the name of the address scope.'))
        parser.add_argument(
            'ip_version',
            metavar='IP_VERSION',
            type=int,
            choices=[4, 6],
            help=_('Specify the address family of the address scope.'))

    def args2body(self, parsed_args):
        body = {'name': parsed_args.name,
                'ip_version': parsed_args.ip_version}
        if parsed_args.shared:
            body['shared'] = True
        neutronV20.update_dict(parsed_args, body, ['tenant_id'])
        return {self.resource: body}


class DeleteAddressScope(neutronV20.DeleteCommand):
    """Delete an address scope."""

    resource = 'address_scope'


class UpdateAddressScope(neutronV20.UpdateCommand):
    """Update an address scope."""

    resource = 'address_scope'

    def add_known_arguments(self, parser):
        parser.add_argument('--name',
                            help=_('Updated name of the address scope.'))
        utils.add_boolean_argument(
            parser, '--shared',
            help=_('Set sharing of address scope. '
                   '(True means shared)'))

    def args2body(self, parsed_args):
        body = {}
        neutronV20.update_dict(parsed_args, body, ['name', 'shared'])
        return {self.resource: body}



# -*- coding: utf-8 -*-
#

# -- General configuration ---------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'reno.sphinxext',
    'openstackdocstheme',
    'cliff.sphinxext',
]

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/python-neutronclient'
openstackdocs_pdf_link = True
openstackdocs_bug_project = 'python-neutronclient'
openstackdocs_bug_tag = 'doc'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
copyright = u'OpenStack Foundation'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output ---------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'openstackdocs'

# Output file base name for HTML help builder.
htmlhelp_basename = 'neutronclientdoc'

# -- Options for LaTeX output ------------------------------------------------

latex_documents = [
    ('index', 'doc-python-neutronclient.tex',
     u'python-neutronclient Documentation',
     u'Neutron Contributors', 'manual'),
]

# Disable usage of xindy https://bugzilla.redhat.com/show_bug.cgi?id=1643664
latex_use_xindy = False

latex_domain_indices = False

latex_elements = {
    'makeindex': '',
    'printindex': '',
    'preamble': r'\setcounter{tocdepth}{5}',
}

# -- Options for cliff.sphinxext plugin ---------------------------------------

autoprogram_cliff_application = 'openstack'



# Copyright 2016 ZTE Corporation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import copy
from unittest import mock
from unittest.mock import call

from osc_lib.cli import format_columns
from osc_lib import exceptions
from osc_lib.tests import utils as tests_utils
import testtools

from neutronclient.osc.v2.trunk import network_trunk as trunk
from neutronclient.osc.v2 import utils as v2_utils
from neutronclient.tests.unit.osc.v2 import fakes as test_fakes
from neutronclient.tests.unit.osc.v2.trunk import fakes


def _get_id(client, id_or_name, resource):
    return id_or_name


class TestCreateNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
    # The new trunk created
    _trunk = fakes.FakeTrunk.create_one_trunk()

    columns = (
        'admin_state_up',
        'description',
        'id',
        'name',
        'port_id',
        'project_id',
        'status',
        'sub_ports',
    )

    def get_data(self):
        return (
            v2_utils.AdminStateColumn(self._trunk['admin_state_up']),
            self._trunk['description'],
            self._trunk['id'],
            self._trunk['name'],
            self._trunk['port_id'],
            self._trunk['project_id'],
            self._trunk['status'],
            format_columns.ListDictColumn(self._trunk['sub_ports']),
        )

    def setUp(self):
        super(TestCreateNetworkTrunk, self).setUp()
        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.create_trunk = mock.Mock(
            return_value={trunk.TRUNK: self._trunk})
        self.data = self.get_data()

        # Get the command object to test
        self.cmd = trunk.CreateNetworkTrunk(self.app, self.namespace)

    def test_create_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_create_default_options(self):
        arglist = [
            "--parent-port", self._trunk['port_id'],
            self._trunk['name'],
        ]
        verifylist = [
            ('parent_port', self._trunk['port_id']),
            ('name', self._trunk['name']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_trunk.assert_called_once_with({
            trunk.TRUNK: {'name': self._trunk['name'],
                          'admin_state_up': self._trunk['admin_state_up'],
                          'port_id': self._trunk['port_id']}
        })
        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_full_options(self):
        self._trunk['description'] = 'foo description'
        self.data = self.get_data()
        subport = self._trunk['sub_ports'][0]
        arglist = [
            "--disable",
            "--description", self._trunk['description'],
            "--parent-port", self._trunk['port_id'],
            "--subport", 'port=%(port)s,segmentation-type=%(seg_type)s,'
            'segmentation-id=%(seg_id)s' % {
                'seg_id': subport['segmentation_id'],
                'seg_type': subport['segmentation_type'],
                'port': subport['port_id']},
            self._trunk['name'],
        ]
        verifylist = [
            ('name', self._trunk['name']),
            ('description', self._trunk['description']),
            ('parent_port', self._trunk['port_id']),
            ('add_subports', [{
                'port': subport['port_id'],
                'segmentation-id': str(subport['segmentation_id']),
                'segmentation-type': subport['segmentation_type']}]),
            ('disable', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_trunk.assert_called_once_with({
            trunk.TRUNK: {'name': self._trunk['name'],
                          'description': self._trunk['description'],
                          'admin_state_up': False,
                          'sub_ports': [subport],
                          'port_id': self._trunk['port_id']}
        })
        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_trunk_with_subport_invalid_segmentation_id_fail(self):
        subport = self._trunk['sub_ports'][0]
        arglist = [
            "--parent-port", self._trunk['port_id'],
            "--subport", "port=%(port)s,segmentation-type=%(seg_type)s,"
            "segmentation-id=boom" % {
                'seg_type': subport['segmentation_type'],
                'port': subport['port_id']},
            self._trunk['name'],
        ]
        verifylist = [
            ('name', self._trunk['name']),
            ('parent_port', self._trunk['port_id']),
            ('add_subports', [{
                'port': subport['port_id'],
                'segmentation-id': 'boom',
                'segmentation-type': subport['segmentation_type']}]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        with testtools.ExpectedException(exceptions.CommandError) as e:
            self.cmd.take_action(parsed_args)
            self.assertEqual("Segmentation-id 'boom' is not an integer",
                             str(e))

    def test_create_network_trunk_subports_without_optional_keys(self):
        subport = copy.copy(self._trunk['sub_ports'][0])
        # Pop out the segmentation-id and segmentation-type
        subport.pop('segmentation_type')
        subport.pop('segmentation_id')
        arglist = [
            '--parent-port', self._trunk['port_id'],
            '--subport', 'port=%(port)s' % {'port': subport['port_id']},
            self._trunk['name'],
        ]
        verifylist = [
            ('name', self._trunk['name']),
            ('parent_port', self._trunk['port_id']),
            ('add_subports', [{
                'port': subport['port_id']}]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_trunk.assert_called_once_with({
            trunk.TRUNK: {'name': self._trunk['name'],
                          'admin_state_up': True,
                          'sub_ports': [subport],
                          'port_id': self._trunk['port_id']}
        })
        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_network_trunk_subports_without_required_key_fail(self):
        subport = self._trunk['sub_ports'][0]
        arglist = [
            '--parent-port', self._trunk['port_id'],
            '--subport', 'segmentation-type=%(seg_type)s,'
            'segmentation-id=%(seg_id)s' % {
                'seg_id': subport['segmentation_id'],
                'seg_type': subport['segmentation_type']},
            self._trunk['name'],
        ]
        verifylist = [
            ('name', self._trunk['name']),
            ('parent_port', self._trunk['port_id']),
            ('add_subports', [{
                'segmentation-id': str(subport['segmentation_id']),
                'segmentation-type': subport['segmentation_type']}]),
        ]

        with testtools.ExpectedException(argparse.ArgumentTypeError):
            self.check_parser(self.cmd, arglist, verifylist)


class TestDeleteNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
    # The trunk to be deleted.
    _trunks = fakes.FakeTrunk.create_trunks(count=2)

    def setUp(self):
        super(TestDeleteNetworkTrunk, self).setUp()

        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.delete_trunk = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = trunk.DeleteNetworkTrunk(self.app, self.namespace)

    def test_delete_trunk(self):
        arglist = [
            self._trunks[0]['name'],
        ]
        verifylist = [
            ('trunk', [self._trunks[0]['name']]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.neutronclient.delete_trunk.assert_called_once_with(
            self._trunks[0]['name'])
        self.assertIsNone(result)

    def test_delete_trunk_multiple(self):
        arglist = []
        verifylist = []

        for t in self._trunks:
            arglist.append(t['name'])
        verifylist = [
            ('trunk', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        calls = []
        for t in self._trunks:
            calls.append(call(t['name']))
        self.neutronclient.delete_trunk.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_delete_trunk_multiple_with_exception(self):
        arglist = [
            self._trunks[0]['name'],
            'unexist_trunk',
        ]
        verifylist = [
            ('trunk',
             [self._trunks[0]['name'], 'unexist_trunk']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = [self._trunks[0], exceptions.CommandError]
        trunk._get_id = (
            mock.Mock(side_effect=get_mock_result)
        )
        with testtools.ExpectedException(exceptions.CommandError) as e:
            self.cmd.take_action(parsed_args)
            self.assertEqual('1 of 2 trunks failed to delete.', str(e))
        self.neutronclient.delete_trunk.assert_called_once_with(
            self._trunks[0]
        )


class TestShowNetworkTrunk(test_fakes.TestNeutronClientOSCV2):

    # The trunk to set.
    _trunk = fakes.FakeTrunk.create_one_trunk()

    columns = (
        'admin_state_up',
        'description',
        'id',
        'name',
        'port_id',
        'project_id',
        'status',
        'sub_ports',
    )
    data = (
        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
        _trunk['description'],
        _trunk['id'],
        _trunk['name'],
        _trunk['port_id'],
        _trunk['project_id'],
        _trunk['status'],
        format_columns.ListDictColumn(_trunk['sub_ports']),
    )

    def setUp(self):
        super(TestShowNetworkTrunk, self).setUp()

        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.show_trunk = mock.Mock(
            return_value={trunk.TRUNK: self._trunk})

        # Get the command object to test
        self.cmd = trunk.ShowNetworkTrunk(self.app, self.namespace)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_show_all_options(self):
        arglist = [
            self._trunk['id'],
        ]
        verifylist = [
            ('trunk', self._trunk['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.neutronclient.show_trunk.assert_called_once_with(
            self._trunk['id'])
        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestListNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
    # Create trunks to be listed.
    _trunks = fakes.FakeTrunk.create_trunks(
        {'created_at': '2001-01-01 00:00:00',
         'updated_at': '2001-01-01 00:00:00'}, count=3)

    columns = (
        'ID',
        'Name',
        'Parent Port',
        'Description'
    )
    columns_long = columns + (
        'Status',
        'State',
        'Created At',
        'Updated At'
    )
    data = []
    for t in _trunks:
        data.append((
            t['id'],
            t['name'],
            t['port_id'],
            t['description']
        ))
    data_long = []
    for t in _trunks:
        data_long.append((
            t['id'],
            t['name'],
            t['port_id'],
            t['description'],
            t['status'],
            v2_utils.AdminStateColumn(t['admin_state_up']),
            '2001-01-01 00:00:00',
            '2001-01-01 00:00:00',
        ))

    def setUp(self):
        super(TestListNetworkTrunk, self).setUp()
        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.list_trunks = mock.Mock(
            return_value={trunk.TRUNKS: self._trunks})

        # Get the command object to test
        self.cmd = trunk.ListNetworkTrunk(self.app, self.namespace)

    def test_trunk_list_no_option(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_trunks.assert_called_once_with()
        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_trunk_list_long(self):
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_trunks.assert_called_once_with()
        self.assertEqual(self.columns_long, columns)
        self.assertListItemEqual(self.data_long, list(data))


class TestSetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
    # Create trunks to be listed.
    _trunk = fakes.FakeTrunk.create_one_trunk()

    columns = (
        'admin_state_up',
        'id',
        'name',
        'description',
        'port_id',
        'project_id',
        'status',
        'sub_ports',
    )
    data = (
        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
        _trunk['id'],
        _trunk['name'],
        _trunk['description'],
        _trunk['port_id'],
        _trunk['project_id'],
        _trunk['status'],
        format_columns.ListDictColumn(_trunk['sub_ports']),
    )

    def setUp(self):
        super(TestSetNetworkTrunk, self).setUp()
        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.update_trunk = mock.Mock(
            return_value={trunk.TRUNK: self._trunk})
        self.neutronclient.trunk_add_subports = mock.Mock(
            return_value=self._trunk)

        # Get the command object to test
        self.cmd = trunk.SetNetworkTrunk(self.app, self.namespace)

    def _test_set_network_trunk_attr(self, attr, value):
        arglist = [
            '--%s' % attr, value,
            self._trunk[attr],
        ]
        verifylist = [
            (attr, value),
            ('trunk', self._trunk[attr]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            attr: value,
        }
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk[attr], {trunk.TRUNK: attrs})
        self.assertIsNone(result)

    def test_set_network_trunk_name(self):
        self._test_set_network_trunk_attr('name', 'trunky')

    def test_test_set_network_trunk_description(self):
        self._test_set_network_trunk_attr('description', 'description')

    def test_set_network_trunk_admin_state_up_disable(self):
        arglist = [
            '--disable',
            self._trunk['name'],
        ]
        verifylist = [
            ('disable', True),
            ('trunk', self._trunk['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'admin_state_up': False,
        }
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk['name'], {trunk.TRUNK: attrs})
        self.assertIsNone(result)

    def test_set_network_trunk_admin_state_up_enable(self):
        arglist = [
            '--enable',
            self._trunk['name'],
        ]
        verifylist = [
            ('enable', True),
            ('trunk', self._trunk['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'admin_state_up': True,
        }
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk['name'], {trunk.TRUNK: attrs})
        self.assertIsNone(result)

    def test_set_network_trunk_nothing(self):
        arglist = [self._trunk['name'], ]
        verifylist = [('trunk', self._trunk['name']), ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {}
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk['name'], {trunk.TRUNK: attrs})
        self.assertIsNone(result)

    def test_set_network_trunk_subports(self):
        subport = self._trunk['sub_ports'][0]
        arglist = [
            '--subport', 'port=%(port)s,segmentation-type=%(seg_type)s,'
            'segmentation-id=%(seg_id)s' % {
                'seg_id': subport['segmentation_id'],
                'seg_type': subport['segmentation_type'],
                'port': subport['port_id']},
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('set_subports', [{
                'port': subport['port_id'],
                'segmentation-id': str(subport['segmentation_id']),
                'segmentation-type': subport['segmentation_type']}]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.neutronclient.trunk_add_subports.assert_called_once_with(
            self._trunk['name'], {'sub_ports': [subport]}
        )
        self.assertIsNone(result)

    def test_set_network_trunk_subports_without_optional_keys(self):
        subport = copy.copy(self._trunk['sub_ports'][0])
        # Pop out the segmentation-id and segmentation-type
        subport.pop('segmentation_type')
        subport.pop('segmentation_id')
        arglist = [
            '--subport', 'port=%(port)s' % {'port': subport['port_id']},
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('set_subports', [{
                'port': subport['port_id']}]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.neutronclient.trunk_add_subports.assert_called_once_with(
            self._trunk['name'], {'sub_ports': [subport]}
        )
        self.assertIsNone(result)

    def test_set_network_trunk_subports_without_required_key_fail(self):
        subport = self._trunk['sub_ports'][0]
        arglist = [
            '--subport', 'segmentation-type=%(seg_type)s,'
            'segmentation-id=%(seg_id)s' % {
                'seg_id': subport['segmentation_id'],
                'seg_type': subport['segmentation_type']},
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('set_subports', [{
                'segmentation-id': str(subport['segmentation_id']),
                'segmentation-type': subport['segmentation_type']}]),
        ]

        with testtools.ExpectedException(argparse.ArgumentTypeError):
            self.check_parser(self.cmd, arglist, verifylist)

        self.neutronclient.trunk_add_subports.assert_not_called()

    def test_set_trunk_attrs_with_exception(self):
        arglist = [
            '--name', 'reallylongname',
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('name', 'reallylongname'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.neutronclient.update_trunk = (
            mock.Mock(side_effect=exceptions.CommandError)
        )
        with testtools.ExpectedException(exceptions.CommandError) as e:
            self.cmd.take_action(parsed_args)
            self.assertEqual(
                "Failed to set trunk '%s': " % self._trunk['name'],
                str(e))
        attrs = {'name': 'reallylongname'}
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk['name'], {trunk.TRUNK: attrs})
        self.neutronclient.trunk_add_subports.assert_not_called()

    def test_set_trunk_add_subport_with_exception(self):
        arglist = [
            '--subport', 'port=invalid_subport',
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('set_subports', [{'port': 'invalid_subport'}]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.neutronclient.trunk_add_subports = (
            mock.Mock(side_effect=exceptions.CommandError)
        )
        with testtools.ExpectedException(exceptions.CommandError) as e:
            self.cmd.take_action(parsed_args)
            self.assertEqual(
                "Failed to add subports to trunk '%s': " % self._trunk['name'],
                str(e))
        self.neutronclient.update_trunk.assert_called_once_with(
            self._trunk['name'], {trunk.TRUNK: {}})
        self.neutronclient.trunk_add_subports.assert_called_once_with(
            self._trunk['name'],
            {'sub_ports': [{'port_id': 'invalid_subport'}]}
        )


class TestListNetworkSubport(test_fakes.TestNeutronClientOSCV2):

    _trunk = fakes.FakeTrunk.create_one_trunk()
    _subports = _trunk['sub_ports']

    columns = (
        'Port',
        'Segmentation Type',
        'Segmentation ID',
    )
    data = []
    for s in _subports:
        data.append((
            s['port_id'],
            s['segmentation_type'],
            s['segmentation_id'],
        ))

    def setUp(self):
        super(TestListNetworkSubport, self).setUp()
        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.trunk_get_subports = mock.Mock(
            return_value={trunk.SUB_PORTS: self._subports})

        # Get the command object to test
        self.cmd = trunk.ListNetworkSubport(self.app, self.namespace)

    def test_subport_list(self):
        arglist = [
            '--trunk', self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.neutronclient.trunk_get_subports.assert_called_once_with(
            self._trunk['name'])
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestUnsetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):

    _trunk = fakes.FakeTrunk.create_one_trunk()

    columns = (
        'admin_state_up',
        'id',
        'name',
        'port_id',
        'project_id',
        'status',
        'sub_ports',
    )
    data = (
        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
        _trunk['id'],
        _trunk['name'],
        _trunk['port_id'],
        _trunk['project_id'],
        _trunk['status'],
        format_columns.ListDictColumn(_trunk['sub_ports']),
    )

    def setUp(self):
        super(TestUnsetNetworkTrunk, self).setUp()

        mock.patch('neutronclient.osc.v2.trunk.network_trunk._get_id',
                   new=_get_id).start()
        self.neutronclient.trunk_remove_subports = mock.Mock(
            return_value=None)

        # Get the command object to test
        self.cmd = trunk.UnsetNetworkTrunk(self.app, self.namespace)

    def test_unset_network_trunk_subport(self):
        subport = self._trunk['sub_ports'][0]
        arglist = [
            "--subport", subport['port_id'],
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
            ('unset_subports', [subport['port_id']]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.neutronclient.trunk_remove_subports.assert_called_once_with(
            self._trunk['name'],
            {trunk.SUB_PORTS: [{'port_id': subport['port_id']}]}
        )
        self.assertIsNone(result)

    def test_unset_subport_no_arguments_fail(self):
        arglist = [
            self._trunk['name'],
        ]
        verifylist = [
            ('trunk', self._trunk['name']),
        ]
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)



# Copyright 2016 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import collections
import copy
from unittest import mock

from oslo_utils import uuidutils


class FakeFWaaS(object):

    def create(self, attrs={}):
        """Create a fake fwaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A OrderedDict faking the fwaas resource
        """
        self.ordered.update(attrs)
        return copy.deepcopy(self.ordered)

    def bulk_create(self, attrs=None, count=2):
        """Create multiple fake fwaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of fwaas resources to fake
        :return:
            A list of dictionaries faking the fwaas resources
        """
        return [self.create(attrs=attrs) for i in range(0, count)]

    def get(self, attrs=None, count=2):
        """Create multiple fake fwaas resources

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of fwaas resources to fake
        :return:
            A list of dictionaries faking the fwaas resource
        """
        if attrs is None:
            self.attrs = self.bulk_create(count=count)
        return mock.Mock(side_effect=attrs)


class FirewallGroup(FakeFWaaS):
    """Fake one or more firewall group"""

    def __init__(self):
        super(FirewallGroup, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'firewall-group-id-' +
             uuidutils.generate_uuid(dashed=False)),
            ('name', 'my-group-' +
             uuidutils.generate_uuid(dashed=False)),
            ('ingress_firewall_policy_id', None),
            ('egress_firewall_policy_id', None),
            ('description', 'my-desc-' +
             uuidutils.generate_uuid(dashed=False)),
            ('status', 'INACTIVE'),
            ('ports', []),
            ('admin_state_up', True),
            ('shared', False),
            ('tenant_id', 'tenant-id-' +
             uuidutils.generate_uuid(dashed=False)),
        ))


class FirewallPolicy(FakeFWaaS):
    """Fake one or more firewall policy"""

    def __init__(self):
        super(FirewallPolicy, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'firewall-policy-' +
             uuidutils.generate_uuid(dashed=False)),
            ('name', 'my-policy-' +
             uuidutils.generate_uuid(dashed=False)),
            ('firewall_rules', []),
            ('description', 'my-desc-' +
             uuidutils.generate_uuid(dashed=False)),
            ('audited', True),
            ('shared', False),
            ('tenant_id', 'tenant-id-' +
             uuidutils.generate_uuid(dashed=False)),
        ))


class FirewallRule(FakeFWaaS):
    """Fake one or more firewall rule"""

    def __init__(self):
        super(FirewallRule, self).__init__()
        self.ordered = collections.OrderedDict((
            ('id', 'firewall-rule-id-' +
             uuidutils.generate_uuid(dashed=False)),
            ('name', 'my-rule-' +
             uuidutils.generate_uuid(dashed=False)),
            ('enabled', False),
            ('description', 'my-desc-' +
             uuidutils.generate_uuid(dashed=False)),
            ('ip_version', 4),
            ('action', 'deny'),
            ('protocol', None),
            ('source_ip_address', '192.168.1.0/24'),
            ('source_port', '1:11111'),
            ('destination_ip_address', '192.168.2.2'),
            ('destination_port', '2:22222'),
            ('shared', False),
            ('tenant_id', 'tenant-id-' +
             uuidutils.generate_uuid(dashed=False)),
            ('source_firewall_group_id', 'firewall-group-id-' +
             uuidutils.generate_uuid(dashed=False)),
            ('destination_firewall_group_id', 'firewall-group-id-' +
             uuidutils.generate_uuid(dashed=False)),
        ))
